/************************************************************************/
/*                                                                      */
/*   svm_common.h                                                       */
/*                                                                      */
/*   Definitions and functions used in both svm_learn and svm_classify. */
/*                                                                      */
/*   Author: Thorsten Joachims                                          */
/*   Date: 06.07.98                                                     */
/*                                                                      */
/*   Copyright (c) 1998  Universitaet Dortmund - All rights reserved    */
/*                                                                      */
/*   This software is available for non-commercial use only. It must    */
/*   not be modified and distributed without prior permission of the    */
/*   author. The author is not responsible for implications from the    */
/*   use of this software.                                              */
/*                                                                      */
/************************************************************************/

# include <stdio.h>
# include <ctype.h>
# include <signal.h>
# include <math.h>
# include <string.h>
# include <stdlib.h>
# include <sys/times.h>
# include <limits.h>

# define VERSION       "V2.0"
# define VERSION_DATE  "05.08.98"

# define CFLOAT  float       /* the type of float to use for caching */
                             /* kernel evaluations. Using float saves */
                             /* us some memory, but you can use double, too */
# define FNUM    short       /* the type used for storing feature ids */
# define FVAL    float       /* the type used for storing feature values */


typedef struct word {
  FNUM    wnum;	
  FVAL    weight;
} WORD;

typedef struct doc {
  long    docnum;
  double  twonorm_sq;
  WORD    *words;
} DOC;

typedef struct kernel_parm {
  long    kernel_type;   /* 0=linear, 1=poly, 2=rbf, 3=sigmoid, 4=custom */
  long    poly_degree;
  double  rbf_gamma;
  double  coef_lin;
  double  coef_const;
  char    custom[50];    /* for user supplied kernel */
} KERNEL_PARM;

typedef struct model {
  long    sv_num;	
  double  b;
  DOC     **supvec;
  double  *alpha;
} MODEL;

double classify_example(MODEL *, DOC *, KERNEL_PARM *);
CFLOAT kernel(KERNEL_PARM *, DOC *, DOC *); 
double custom_kernel(KERNEL_PARM *, DOC *, DOC *); 
double sprod_ss(WORD *, WORD *);
double model_length_s(MODEL *, KERNEL_PARM *);
void read_documents(char *, DOC *, long *, long, long, long *, long *);
void parse_document(char *, DOC *, long *, long *, long);
void nol_ll(char *, long *, long *, long *);
long min(long, long);
long max(long, long);
long get_runtime();
void *my_malloc(long); 
void copyright_notice();

long verbosity;              /* verbosity level (0-3) */
long kernel_cache_statistic;


