/***********************************************************************/
/*                                                                     */
/*   svm_loqo.c                                                        */
/*                                                                     */
/*   Interface to the PR_LOQO optimization package for SVM.            */
/*                                                                     */
/*   Author: Thorsten Joachims                                         */
/*   Date: 06.07.98                                                    */
/*                                                                     */
/*   Copyright (c) 1998  Universitaet Dortmund - All rights reserved   */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/

# include <math.h>
# include "pr_loqo/pr_loqo.h"

/* Common Block Declarations */

long verbosity;

/* /////////////////////////////////////////////////////////////// */

int nx = 400; /* Maximum size of QP-subproblems */

/* /////////////////////////////////////////////////////////////// */

double *optimize_qp();
double svm_c;
double *opt_ce,*opt_ce0,*opt_g,*opt_g0,*opt_xinit;
long   opt_n;
double opt_precision;
double *primal=0,*dual=0,*opt_low,*opt_up;

/* /////////////////////////////////////////////////////////////// */

void *my_malloc();
long max();

double *optimize_qp()
/* start the optimizer and return the optimal values */
{
  long i,j,result;
  double margin;
  double sigdig;
  int iter,eqcount;

  if(!primal) { /* allocate memory at first call */
    primal=(double *)my_malloc(sizeof(double)*nx*3);
    dual=(double *)my_malloc(sizeof(double)*(nx*2+1));
    opt_low=(double *)my_malloc(sizeof(double)*nx);
    opt_up=(double *)my_malloc(sizeof(double)*nx);
    for(i=0;i<nx;i++) {
      opt_low[i]=0;
      opt_up[i]=svm_c;
    }
  }

  if(verbosity>=3) {
    printf("\n\n");
    for(i=0;i<opt_n;i++) {
      printf("%f: ",opt_g0[i]);
      for(j=0;j<opt_n;j++) {
	printf("%f ",opt_g[i*opt_n+j]);
      }
      printf(": a=%.30f",opt_xinit[i]);
      printf(": y=%f\n",opt_ce[i]);
    }
    printf("\n");
  }

  opt_ce0[0]*=(-1.0);
  result=STILL_RUNNING;
  for(margin=0.15,sigdig=-log10(opt_precision),iter=10000;
      (margin<=1.0) && (result!=OPTIMAL_SOLUTION);
      margin+=0.1,sigdig-=1.0,iter+=1000) {
    result=pr_loqo((int)opt_n,(int)1,(double *)opt_g0,(double *)opt_g,
		   (double *)opt_ce,(double *)opt_ce0,
		   (double *)opt_low,(double *)opt_up,
		   (double *)primal,(double *)dual, 
		   (int)max((long)(result!=STILL_RUNNING),verbosity-2),
		   (double)sigdig,(int)iter, 
		   (double)margin,(double)svm_c/4.0,(int)0);
    if(result!=OPTIMAL_SOLUTION) {
      printf("\nWARNING: PR_LOQO did not converge. Restarting...\n");
      printf("\nsigdig=%f\n",sigdig);
    }
  }

  eqcount=0;
  for(i=0;i<opt_n;i++) { /* check, whether there is progress to avoid loops */
    if(primal[i]==opt_xinit[i]) eqcount++;
  }
  if(eqcount==opt_n) {
    opt_precision/=10;  /* increase precision */
    printf("\nWARNING: Increasing precision. Restarting...\n");
  }

  if(result!=OPTIMAL_SOLUTION) {
    printf("\nERROR: PR_LOQO did not converge. \n");
    return(opt_xinit);
  }
  else {
    return(primal);
  }
}

